package com.ruoyi.jjh.declaration.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.jjh.declaration.entity.BmsDeclarationRecords;
import com.ruoyi.jjh.declaration.entity.BmsTemplateInfo;
import com.ruoyi.jjh.declaration.service.IBmsDeclarationRecordsService;
import com.ruoyi.jjh.declaration.service.IBmsTemplateInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * 申报模板信息Controller
 *
 * @author farben
 * @date 2023-08-25
 */
@RestController
@Api(tags = "申报模板")
@RequestMapping("/system/templateInfo")
public class BmsTemplateInfoController extends BaseController {
    @Autowired
    private IBmsTemplateInfoService bmsTemplateInfoService;

    @Resource
    private IBmsDeclarationRecordsService iBmsDeclarationRecordsService;
    /**
     * 查询申报模板信息列表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @GetMapping("/list")
    @ApiOperation(value = "查询申报模板信息列表", response = BmsTemplateInfo.class)
    public AjaxResult list(Page<BmsTemplateInfo> page, BmsTemplateInfo bmsTemplateInfo) {
        return success(bmsTemplateInfoService.selectBmsTemplateInfoList(page, bmsTemplateInfo));
    }

    /**
     * 新增项目时查询所有模板信息
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @GetMapping("/allList")
    @ApiOperation(value = "新增项目时查询所有模板信息", response = BmsTemplateInfo.class)
    public AjaxResult allList(BmsTemplateInfo bmsTemplateInfo) {
        bmsTemplateInfo.setStatus(1);
        bmsTemplateInfo.setIsDeleted(0);
        return success(bmsTemplateInfoService.selectBmsTemplateInfoList(bmsTemplateInfo));
    }

    /**
     * 导出申报模板信息列表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @Log(title = "申报模板信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BmsTemplateInfo bmsTemplateInfo) {
        List<BmsTemplateInfo> list = bmsTemplateInfoService.selectBmsTemplateInfoList(bmsTemplateInfo);
        ExcelUtil<BmsTemplateInfo> util = new ExcelUtil<>(BmsTemplateInfo.class);
        util.exportExcel(response, list, "申报模板信息数据");
    }

    /**
     * 导出模板表单
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @Log(title = "导出模板表单", businessType = BusinessType.EXPORT)
    @ApiOperation(value = "导出模板表单")
    @GetMapping(value = "/exportTemplate/{id}")
    public void exportTemplate(HttpServletRequest request, HttpServletResponse response, @PathVariable("id") Long id) {
        bmsTemplateInfoService.exportTemplate(request, response, id);
    }

    /**
     * 获取申报模板信息详细信息
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,ent')")
    @GetMapping(value = "/{id}")
    @ApiOperation(value = "获取申报模板信息详细信息")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bmsTemplateInfoService.selectByIdOne(id));
    }

    /**
     * 新增申报模板信息
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @Log(title = "新增申报模板信息", businessType = BusinessType.INSERT)
    @ApiOperation(value = "新增申报模板信息")
    @PostMapping
    public AjaxResult add(@RequestBody @Valid BmsTemplateInfo bmsTemplateInfo) {
        return toAjax(bmsTemplateInfoService.add(bmsTemplateInfo));
    }

    /**
     * 修改申报模板信息
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @Log(title = "修改申报模板信息", businessType = BusinessType.UPDATE)
    @ApiOperation(value = "修改申报模板信息")
    @PostMapping("/edit")
    public AjaxResult edit(@RequestBody BmsTemplateInfo bmsTemplateInfo) {
        Long count = iBmsDeclarationRecordsService.lambdaQuery().eq(BmsDeclarationRecords::getTemplateId,bmsTemplateInfo.getId()).count();
        if(count>0){
            throw new ServiceException("无法修改!");
        }else {
            return toAjax(bmsTemplateInfoService.updateById(bmsTemplateInfo));
        }
    }

    /**
     * 删除申报模板信息
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @Log(title = "申报模板信息", businessType = BusinessType.DELETE)
    @ApiOperation(value = "删除申报模板信息")
    @PostMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(bmsTemplateInfoService.deleteBmsTemplateInfoByIds(ids));
    }
}
