package com.ruoyi.jjh.declaration.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.jjh.declaration.entity.dto.BmsTemplateRecordAddDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsTemplateRecordQueryDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsTemplateRecordUpdateDto;
import com.ruoyi.jjh.declaration.entity.vo.BmsTemplateRecordQueryVo;
import com.ruoyi.jjh.declaration.service.IBmsTemplateRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 在线模板Controller
 *
 * @author farben
 * @date 2023-09-07
 */
@RestController
@RequestMapping("/system/templateRecord")
@Api(tags = "在线模板记录")
public class BmsTemplateRecordController extends BaseController {
    @Autowired
    private IBmsTemplateRecordService bmsTemplateRecordService;

    /**
     * 分页查询在线模板列表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @GetMapping("/list")
    @ApiOperation(value = "分页查询在线模板列表", response = BmsTemplateRecordQueryVo.class)
    public AjaxResult list(Page<BmsTemplateRecordQueryVo> page, BmsTemplateRecordQueryDto bmsTemplateRecordQueryDto) {
        return success(bmsTemplateRecordService.selectBmsTemplateRecordList(page, bmsTemplateRecordQueryDto));
    }

//    /**
//     * 企业端选择表单模板列表
//     * @param bmsTemplateQueryDto
//     * @return
//     */
//    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
//    @ApiOperation(value = "企业端选择表单模板列表")
//    @GetMapping("/templateNameList")
//    public TableDataInfo templateNameList(BmsTemplateQueryDto bmsTemplateQueryDto){
//        List<BmsTemplateRecordQueryVo> list = bmsTemplateRecordService.selectBmsTemplateNameList(bmsTemplateQueryDto);
//        return getDataTable(list);
//    }


    /**
     * 补录
     *
     * @param bmsTemplateRecordUpdateDto
     * @return
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "补录")
    @PostMapping("/supplementation")
    public AjaxResult supplementation(@RequestBody BmsTemplateRecordUpdateDto bmsTemplateRecordUpdateDto) {
        return toAjax(bmsTemplateRecordService.supplementation(bmsTemplateRecordUpdateDto));
    }

    /**
     * 导出在线模板列表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @Log(title = "在线模板", businessType = BusinessType.EXPORT)
    @ApiOperation(value = "导出在线模板列表")
    @PostMapping("/export")
    public void export(HttpServletResponse response, BmsTemplateRecordQueryDto bmsTemplateRecordQueryDto) {
        List<BmsTemplateRecordQueryVo> list = bmsTemplateRecordService.selectBmsTemplateRecordList(bmsTemplateRecordQueryDto);
        ExcelUtil<BmsTemplateRecordQueryVo> util = new ExcelUtil<BmsTemplateRecordQueryVo>(BmsTemplateRecordQueryVo.class);
        util.exportExcel(response, list, "在线模板数据");
    }

    /**
     * 获取在线模板详细信息
     */
    @ApiOperation(value = "获取在线模板详细信息")
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,ent')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bmsTemplateRecordService.selectBmsTemplateRecordById(id));
    }

    /**
     * 新增在线模板
     */
    @ApiOperation(value = "新增在线模板")
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov')")
    @Log(title = "在线模板", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BmsTemplateRecordAddDto bmsTemplateRecordAddDto) {
        return toAjax(bmsTemplateRecordService.insertBmsTemplateRecord(bmsTemplateRecordAddDto));
    }


//    /**
//     * 修改在线模板
//     */
//    @ApiOperation(value = "修改在线模板")
//    @PreAuthorize("@ss.hasAnyRoles('admin')")
//    @Log(title = "在线模板", businessType = BusinessType.UPDATE)
//    @PostMapping("/edit")
//    public AjaxResult edit(@RequestBody BmsTemplateRecordUpdateDto bmsTemplateRecordUpdateDto) {
//        return toAjax(bmsTemplateRecordService.updateBmsTemplateRecord(bmsTemplateRecordUpdateDto));
//    }

    /**
     * 删除在线模板
     */
    @ApiOperation(value = "删除在线模板")
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @Log(title = "在线模板", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(bmsTemplateRecordService.deleteBmsTemplateRecordByIds(ids));
    }
}
