package com.ruoyi.jjh.declaration.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.ruoyi.jjh.declaration.entity.BmsDeclarationRecords;
import com.ruoyi.jjh.declaration.entity.dto.ApprovalDeclarationRecordsQueryDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsDeclarationRecordsDto;
import com.ruoyi.jjh.declaration.entity.vo.ApprovalDeclarationRecordsQueryVo;
import com.ruoyi.jjh.declaration.entity.vo.BmsDeclarationRecordsQueryVo;
import com.ruoyi.jjh.ent.entity.JProject;
import com.ruoyi.jjh.ent.entity.JSmartDeclaration;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 在线申报记录Mapper接口
 *
 * @author farben
 * @date 2023-08-28
 */
public interface BmsDeclarationRecordsMapper extends BaseMapper<BmsDeclarationRecords> {
    /**
     * 查询在线申报记录
     *
     * @param id 在线申报记录主键
     * @return 在线申报记录
     */
    public BmsDeclarationRecords selectBmsDeclarationRecordsById(Long id);

    /**
     * 查询在线申报记录列表
     *
     * @param bmsDeclarationRecordsDto 在线申报记录
     * @return 在线申报记录集合
     */
    public List<BmsDeclarationRecordsQueryVo> selectBmsDeclarationRecordsList(BmsDeclarationRecordsDto bmsDeclarationRecordsDto);

    public List<BmsDeclarationRecordsQueryVo> selectBmsDeclarationRecordsZwList(BmsDeclarationRecordsDto bmsDeclarationRecordsDto);


    /**
     * 审核列表
     */
    public List<ApprovalDeclarationRecordsQueryVo> approvalList(ApprovalDeclarationRecordsQueryDto approvalDeclarationRecordsQueryDto);

    /**
     * 新增在线申报记录
     *
     * @param bmsDeclarationRecords 在线申报记录
     * @return 结果
     */
    public int insertBmsDeclarationRecords(BmsDeclarationRecords bmsDeclarationRecords);

    /**
     * 修改在线申报记录
     *
     * @param bmsDeclarationRecords 在线申报记录
     * @return 结果
     */
    public int updateBmsDeclarationRecords(BmsDeclarationRecords bmsDeclarationRecords);

    /**
     * 删除在线申报记录
     *
     * @param id 在线申报记录主键
     * @return 结果
     */
    public int deleteBmsDeclarationRecordsById(Long id);

    /**
     * 批量删除在线申报记录
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteBmsDeclarationRecordsByIds(Long[] ids);

    /**
     * 更新项目名称by Id
     *
     * @param id
     * @param projectName
     * @return {@link int}
     * @author emiya.xie
     * @create 2023/9/6 15:13
     */
    public int updateProjectNameById(@Param("id") Long id, @Param("projectName") String projectName, @Param("projectId") Long projectId);

    List<BmsDeclarationRecordsQueryVo> selectDeclarationRecordsToIdList(@Param("list") List<Long> list);

    /**
     * 根据企业id和模板id查询
     *
     * @return
     */
    List<BmsDeclarationRecords> selectET(@Param("templateId") Long templateId, @Param("list") List<String> list);


    /**
     * 申报模板不限时并且未填报的
     *
     * @return
     */
    List<JSmartDeclaration> allNoLimited();

    /**
     * 获取某个在线记录id可以添加到项目库的数据
     *
     * @return
     */
    JProject getAddProject(Long id);

}
