package com.ruoyi.jjh.declaration.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.jjh.declaration.entity.BmsTemplateInfo;
import com.ruoyi.jjh.declaration.mapper.BmsTemplateInfoMapper;
import com.ruoyi.jjh.declaration.service.IBmsEnterpriseDirectoryService;
import com.ruoyi.jjh.declaration.service.IBmsTemplateInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDate;
import java.util.List;

/**
 * 申报模板信息Service业务层处理
 *
 * @author farben
 * @date 2023-08-25
 */
@Service
public class BmsTemplateInfoServiceImpl extends ServiceImpl<BmsTemplateInfoMapper, BmsTemplateInfo> implements IBmsTemplateInfoService {

    @Autowired
    private IBmsEnterpriseDirectoryService enterpriseDirectoryService;

    /**
     * 查询申报模板信息列表
     *
     * @param bmsTemplateInfo 申报模板信息
     * @return 申报模板信息
     */
    @Override
    public Page<BmsTemplateInfo> selectBmsTemplateInfoList(Page<BmsTemplateInfo> page, BmsTemplateInfo bmsTemplateInfo) {
        return baseMapper.templateInfoList(page, bmsTemplateInfo);
    }

    /**
     * 新增项目时查询所有模板信息
     *
     * @param bmsTemplateInfo 申报模板信息
     * @return 申报模板信息集合
     */
    @Override
    public List<BmsTemplateInfo> selectBmsTemplateInfoList(BmsTemplateInfo bmsTemplateInfo) {
        return baseMapper.templateInfoList(bmsTemplateInfo);
    }

    /**
     * 导出模板表单
     *
     * @param request
     * @param response
     * @param id
     * @return
     * @author emiya.xie
     * @create 2023/8/31 10:25
     */
    @Override
    public void exportTemplate(HttpServletRequest request, HttpServletResponse response, Long id) {
        BmsTemplateInfo bmsTemplateInfo = baseMapper.selectById(id);
        if (BeanUtil.isEmpty(bmsTemplateInfo)) {
            throw new ServiceException("没有找到合适的模板！");
        }
        String fileName = bmsTemplateInfo.getTemplateName() + ".xlsx";
        String file = "/template/" + fileName;
        InputStream is = this.getClass().getResourceAsStream(file);
        byte[] fileData = new byte[0];
        try {
            fileData = input2byte(is);
        } catch (IOException e) {
            e.printStackTrace();
        }
        FileUtils.downloadFile(response, request, fileName, fileData);
    }

    private byte[] input2byte(InputStream inStream)
            throws IOException {
        ByteArrayOutputStream swapStream = new ByteArrayOutputStream();
        byte[] buff = new byte[100];
        int rc = 0;
        while ((rc = inStream.read(buff, 0, 100)) > 0) {
            swapStream.write(buff, 0, rc);
        }
        byte[] in2b = swapStream.toByteArray();
        return in2b;
    }


    /**
     * 批量删除申报模板信息
     *
     * @param ids 需要删除的申报模板信息主键
     * @return 结果
     */
    @Override
    public int deleteBmsTemplateInfoByIds(Long[] ids) {
        return baseMapper.deleteBmsTemplateInfoByIds(ids);
    }

    /**
     * 删除申报模板信息信息
     *
     * @param id 申报模板信息主键
     * @return 结果
     */
    @Override
    public int deleteBmsTemplateInfoById(Long id) {
        return baseMapper.deleteBmsTemplateInfoById(id);
    }


    /**
     * 新增申报模板
     */
    @Override
    public Boolean add(BmsTemplateInfo bmsTemplateInfo) {
//        String str ="bms_"+ RandomUtil.randomString(5)+LocalDate.now().getMonthValue()+LocalDate.now().getDayOfMonth();
//        bmsTemplateInfo.setTableName(str);
        changeStatus(bmsTemplateInfo);
        //新增的时候根据当前时间判断状态是否有效
        return save(bmsTemplateInfo);
    }

    /**
     * 查询单个模板
     */
    @Override
    public BmsTemplateInfo selectByIdOne(Long id) {
        return baseMapper.selectByIdOne(id);
    }

    /**
     * 修改申报模板的状态
     */
    @Override
    public void changeStatus(BmsTemplateInfo items) {
        LocalDate currentDate = LocalDate.now();
        LocalDate startDate = LocalDate.of(items.getUseStart().getYear(), items.getUseStart().getMonth(), items.getUseStart().getDayOfMonth());
        LocalDate endDate = LocalDate.of(items.getUseEnd().getYear(), items.getUseEnd().getMonth(), items.getUseEnd().getDayOfMonth());
        if (currentDate.isBefore(startDate)) {
            items.setStatus(0);
        }
        if (currentDate.isAfter(endDate)) {
            items.setStatus(0);
        }
        if (currentDate.equals(startDate) || currentDate.equals(endDate)) {
            items.setStatus(1);
        }
        if (currentDate.isAfter(startDate) && currentDate.isBefore(endDate)) {
            items.setStatus(1);
        }
    }
}
