package com.ruoyi.jjh.ent.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.jjh.ent.entity.JEnterpriseContact;
import com.ruoyi.jjh.ent.entity.request.JEnterpriseContactRequest;
import com.ruoyi.jjh.ent.service.JEnterpriseContactService;
import com.ruoyi.jjh.ent.service.SmsAlertsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.Arrays;

/**
 * 企业端信息维护（j_enterprise_contact）表控制层
 *
 * @author du
 * @since 2024/5/13 9:56
 */
@RestController
@RequestMapping("/jjh/jEnterpriseContact")
@Api(tags = "企业端信息维护")
public class JEnterpriseContactController extends BaseController {

    @Resource
    private JEnterpriseContactService jEnterpriseContactService;

    @Value("${isTiming}")
    private Boolean isTiming;

    @Resource
    private SmsAlertsService smsAlertsService;

    /**
     * 分页查询所有数据
     *
     * @param page                      分页对象
     * @param jEnterpriseContactRequest 查询实体
     * @return 所有数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,ent')")
    @ApiOperation(value = "分页查询所有数据", response = JEnterpriseContact.class)
    @GetMapping
    public AjaxResult selectAll(Page<JEnterpriseContact> page, JEnterpriseContactRequest jEnterpriseContactRequest) {
        // 获取当前登录用户是否是企业端
        boolean admin = false;
        boolean b = false;
        SysUser str = null;
        try {
            admin = SecurityUtils.isAdmin(SecurityUtils.getUserId());
            b = SecurityUtils.hasRole("ent");
            str = SecurityUtils.getLoginUser().getUser();
        } catch (Exception ignored) {
        }
        if (!admin && b) {
            jEnterpriseContactRequest.setEnterpriseName(str.getNickName());
            jEnterpriseContactRequest.setEnterpriseCode(str.getUserName());
        }
        if (page.getCurrent() >= 1 & page.getSize() >= 1) {
            return success(jEnterpriseContactService.page(page, jEnterpriseContactRequest));
        } else {
            return success(jEnterpriseContactService.page(jEnterpriseContactRequest));
        }
    }


    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,ent')")
    @ApiOperation(value = "通过主键查询单条数据", response = JEnterpriseContact.class)
    @GetMapping("{id}")
    public AjaxResult selectOne(@PathVariable Serializable id) {
        return success(jEnterpriseContactService.getById(id));
    }

    /**
     * 新增数据
     *
     * @param jEnterpriseContact 实体对象
     * @return 新增结果
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,ent')")
    @ApiOperation(value = "新增数据")
    @PostMapping
    public AjaxResult insert(@RequestBody JEnterpriseContact jEnterpriseContact) {
        if (isTiming) {
            smsAlertsService.addSsmPhone(jEnterpriseContact.getContactPhone());
        }
        return success(jEnterpriseContactService.save(jEnterpriseContact));
    }

    /**
     * 修改数据
     *
     * @param jEnterpriseContact 实体对象
     * @return 修改结果
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,ent')")
    @ApiOperation(value = "修改数据")
    @PostMapping("/edit")
    public AjaxResult update(@RequestBody JEnterpriseContact jEnterpriseContact) {
        if (isTiming) {
            smsAlertsService.addSsmPhone(jEnterpriseContact.getContactPhone());
        }
        return success(jEnterpriseContactService.updateById(jEnterpriseContact));
    }

    /**
     * 删除数据
     *
     * @param ids 主键结合
     * @return 删除结果
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,ent')")
    @ApiOperation(value = "删除数据")
    @PostMapping("/{ids}")
    public AjaxResult delete(@PathVariable Long[] ids) {
        return success(jEnterpriseContactService.removeByIds(Arrays.asList(ids)));
    }
}
