package com.ruoyi.jjh.ent.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.jjh.declaration.service.IBmsEnterpriseBasicInfoService;
import com.ruoyi.jjh.ent.entity.JProject;
import com.ruoyi.jjh.ent.entity.response.HonorResponse;
import com.ruoyi.jjh.ent.entity.response.JAllIndustryResponse;
import com.ruoyi.jjh.ent.entity.response.JBigMiddleIndustryResponse;
import com.ruoyi.jjh.ent.entity.response.JIndustryDepthResponse;
import com.ruoyi.jjh.ent.entity.response.JServicesDevelopResponse;
import com.ruoyi.jjh.ent.entity.response.ProjectTrackingResponse;
import com.ruoyi.jjh.ent.mapper.JDataScreenMapper;
import com.ruoyi.jjh.ent.service.JContactsService;
import com.ruoyi.jjh.ent.service.JDataScreenService;
import com.ruoyi.jjh.ent.service.JHeadQuartersRevenueService;
import com.ruoyi.jjh.ent.service.JProjectService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 数据大屏业务处理层
 *
 * @author du
 * @since 2024/5/13 9:58
 */
@Service
public class JDataScreenImpl implements JDataScreenService {

    @Resource
    private JDataScreenMapper jDataScreenMapper;

    @Resource
    private JProjectService jProjectService;

    @Resource
    private IBmsEnterpriseBasicInfoService enterpriseBasicInfoService;


    @Resource
    private JHeadQuartersRevenueService jHeadQuartersRevenueService;
    @Resource
    private JContactsService jc;

    /**
     * 荣誉情况
     *
     * @return 所有数据
     */
    @Override
    public List<HonorResponse> getHonor() {
        int x = 0;
        for (JProject jp : jProjectService.lambdaQuery().eq(JProject::getProjectMiddleType, 4).list()) {
            if ("否".equals(JSONObject.parseObject(jp.getOtherJson()).get("是否通过2022年度“331”评价"))) {
                x += 1;
            }
        }
        List<HonorResponse> honor = jDataScreenMapper.getHonor();
        for (HonorResponse y : honor) {
            if (y.getProjectBigType() == 4) {
                y.setCount1(String.valueOf(Integer.parseInt(y.getCount1()) - x));
            }
            if (y.getProjectMiddleType() == 4) {
                y.setCount2(String.valueOf(Integer.parseInt(y.getCount2()) - x));
            }
            if (y.getProjectSmallType() == 6) {
                y.setCount3(String.valueOf(Integer.parseInt(y.getCount3()) - x));
            }
        }
        return honor;
    }

    /**
     * 项目追踪情况
     *
     * @return 所有数据
     */
    @Override
    public List<ProjectTrackingResponse> getProjectTracking() {
        return jDataScreenMapper.getProjectTracking();
    }

    /**
     * 服务业发展总览
     *
     * @return 所有数据
     */
    @Override
    public List<JServicesDevelopResponse> servicesDevelop(Integer type) {
        return jDataScreenMapper.servicesDevelop(type);
    }

    /**
     * 行业纵深情况
     *
     * @return 所有数据
     */
    @Override
    public List<JIndustryDepthResponse> getIndustryDepth() {
        return jDataScreenMapper.getIndustryDepth();
    }

    /**
     * 所有分行业汇总
     *
     * @return 所有数据
     */
    @Override
    public List<JAllIndustryResponse> getAllIndustry() {
        //获取中类数据
        List<JBigMiddleIndustryResponse> list2 = jDataScreenMapper.getMiddleIndustry();
        //获取大类数据
        List<JAllIndustryResponse> all = jDataScreenMapper.getBigIndustry();
        // 向每个 JAllIndustryResponse 对象的 children 字段中添加数据
        all.forEach(allResponse -> {
            List<JAllIndustryResponse> children = new ArrayList<>();
            list2.forEach(middleResponse -> {
                if (allResponse.getNameType().equals(middleResponse.getCategories())) {
                    JAllIndustryResponse child = new JAllIndustryResponse();
                    BeanUtil.copyProperties(middleResponse, child);
                    children.add(child);
                }
            });
            allResponse.setChildren(children);
        });
        return all;
    }

//    /**
//     * 数据库脱敏
//     */
//    @Override
//    public String sqltm() {
//        //修改企业库的企业名称
        //规上企业营收企业加密
//        List<BmsEnterpriseBasicInfo> qyList = enterpriseBasicInfoService.list();
//        List<BmsEnterpriseBasicInfo> neList = new ArrayList<>();
//        for (BmsEnterpriseBasicInfo items : qyList) {
//            String result = items.getQymc().substring(2, items.getQymc().length()-4);
//            BmsEnterpriseBasicInfo be = new BmsEnterpriseBasicInfo();
//            be.setId(items.getId());
//            StringBuilder stringBuilder = new StringBuilder();
//            stringBuilder.append(items.getQymc(), 0, 2);
//            for (int i = 0; i < result.length(); i++) {
//                stringBuilder.append("*");
//            }
//            stringBuilder.append(items.getQymc().substring(items.getQymc().length()-4));
//            be.setQymc(stringBuilder.toString());
//            neList.add(be);
//        }
//        enterpriseBasicInfoService.updateBatchById(neList);

//          //修改企业库的法人姓名
//        List<BmsEnterpriseBasicInfo> qyList = enterpriseBasicInfoService.list();
//        List<BmsEnterpriseBasicInfo> neList = new ArrayList<>();
//        for (BmsEnterpriseBasicInfo items : qyList) {
//            if(items.getFddbrxm()!=null){
//                String result = items.getFddbrxm().substring(1, items.getFddbrxm().length() );
//                BmsEnterpriseBasicInfo be = new BmsEnterpriseBasicInfo();
//                be.setId(items.getId());
//                StringBuilder stringBuilder = new StringBuilder();
//                stringBuilder.append(items.getFddbrxm(), 0, 1);
//                for (int i = 0; i < result.length(); i++) {
//                    stringBuilder.append("*");
//                }
//                be.setFddbrxm(stringBuilder.toString());
//                neList.add(be);
//            }
//        }
//        enterpriseBasicInfoService.updateBatchById(neList);


        //          //修改企业库的法人姓名
//        List<BmsEnterpriseBasicInfo> qyList = enterpriseBasicInfoService.list();
//        List<BmsEnterpriseBasicInfo> neList = new ArrayList<>();
//        for (BmsEnterpriseBasicInfo items : qyList) {
//            if(items.getFddbrxm()!=null){
//                String result = items.getFddbrxm().substring(1, items.getFddbrxm().length() );
//                BmsEnterpriseBasicInfo be = new BmsEnterpriseBasicInfo();
//                be.setId(items.getId());
//                StringBuilder stringBuilder = new StringBuilder();
//                stringBuilder.append(items.getFddbrxm(), 0, 1);
//                for (int i = 0; i < result.length(); i++) {
//                    stringBuilder.append("*");
//                }
//                be.setFddbrxm(stringBuilder.toString());
//                neList.add(be);
//            }
//        }
//        enterpriseBasicInfoService.updateBatchById(neList);

        //修改企业库的手机号
//        List<BmsEnterpriseBasicInfo> qyList = enterpriseBasicInfoService.list();
//        List<BmsEnterpriseBasicInfo> neList = new ArrayList<>();
//        for (BmsEnterpriseBasicInfo items : qyList) {
//            if(items.getLxdh()!=null& !Objects.equals(items.getLxdh(), "")) {
//                if (items.getLxdh().length() >= 10) {
//                    String result = items.getLxdh().substring(3, items.getLxdh().length() - 4);
//                    BmsEnterpriseBasicInfo be = new BmsEnterpriseBasicInfo();
//                    be.setId(items.getId());
//                    StringBuilder stringBuilder = new StringBuilder();
//                    stringBuilder.append(items.getLxdh(), 0, 3);
//                    for (int i = 0; i < result.length(); i++) {
//                        stringBuilder.append("*");
//                    }
//                    stringBuilder.append(items.getLxdh().substring(items.getLxdh().length() - 4));
//                    be.setLxdh(stringBuilder.toString());
//                    neList.add(be);
//                } else if(items.getLxdh().length() >= 4){
//                    String result = items.getLxdh().substring(2, items.getLxdh().length() - 2);
//                    BmsEnterpriseBasicInfo be = new BmsEnterpriseBasicInfo();
//                    be.setId(items.getId());
//                    StringBuilder stringBuilder = new StringBuilder();
//                    stringBuilder.append(items.getLxdh(), 0, 2);
//                    for (int i = 0; i < result.length(); i++) {
//                        stringBuilder.append("*");
//                    }
//                    stringBuilder.append(items.getLxdh().substring(items.getLxdh().length() - 2));
//                    be.setLxdh(stringBuilder.toString());
//                    neList.add(be);
//                }
//            }
//        }
//        enterpriseBasicInfoService.updateBatchById(neList);

        //规上企业营收企业加密
//        List<BmsEnterpriseBasicInfo> jDataScreenMapper1 = jDataScreenMapper.get1();
//        System.out.println(jDataScreenMapper1.size());
//        List<BmsEnterpriseBasicInfo> neList = new ArrayList<>();
//        for (BmsEnterpriseBasicInfo items : jDataScreenMapper1) {
//            String result = items.getQymc().substring(2, items.getQymc().length()-4);
//            BmsEnterpriseBasicInfo be = new BmsEnterpriseBasicInfo();
//            be.setId(items.getId());
//            StringBuilder stringBuilder = new StringBuilder();
//            stringBuilder.append(items.getQymc(), 0, 2);
//            for (int i = 0; i < result.length(); i++) {
//                stringBuilder.append("*");
//            }
//            stringBuilder.append(items.getQymc().substring(items.getQymc().length()-4));
//            be.setQymc(stringBuilder.toString());
//            neList.add(be);
//        }
//        for (BmsEnterpriseBasicInfo i : neList) {
//            jDataScreenMapper.up(i);
//        }
        //联系人修改
//        List<JContacts> list = jc.list();
//        List<JContacts> newList = new ArrayList<>();
//        for (JContacts items : list) {
//            JContacts be = new JContacts();
//            System.out.println(items.getDwAgencykey());
//            be.setDwAgencykey(items.getDwAgencykey());
//            if (items.getQymc() != null) {
//                String result = items.getQymc().substring(2, items.getQymc().length() - 4);
//                StringBuilder stringBuilder = new StringBuilder();
//                stringBuilder.append(items.getQymc(), 0, 2);
//                for (int i = 0; i < result.length(); i++) {
//                    stringBuilder.append("*");
//                }
//                stringBuilder.append(items.getQymc().substring(items.getQymc().length() - 4));
//                be.setQymc(stringBuilder.toString());
//            }
//            if (items.getLlrxm() != null & !Objects.equals(items.getLlrxm(), "")) {
//                String result = items.getLlrxm().substring(1, items.getLlrxm().length());
//                StringBuilder stringBuilder = new StringBuilder();
//                stringBuilder.append(items.getLlrxm(), 0, 1);
//                for (int i = 0; i < result.length(); i++) {
//                    stringBuilder.append("*");
//                }
//                be.setLlrxm(stringBuilder.toString());
//            }
//            if (items.getDzyx() != null & !Objects.equals(items.getDzyx(), "")) {
//                StringBuilder stringBuilder = new StringBuilder();
//                for (int i = 0; i < 4; i++) {
//                    stringBuilder.append("*");
//                }
//                String result = items.getDzyx().substring(4, items.getDzyx().length());
//                stringBuilder.append(result);
//                be.setDzyx(stringBuilder.toString());
//            }
//            if (items.getYddh() != null &!Objects.equals(items.getYddh(), "")) {
//                if (items.getYddh().length() >= 10) {
//                    String result = items.getYddh().substring(3, items.getYddh().length() - 4);
//                    StringBuilder stringBuilder = new StringBuilder();
//                    stringBuilder.append(items.getYddh(), 0, 3);
//                    for (int i = 0; i < result.length(); i++) {
//                        stringBuilder.append("*");
//                    }
//                    stringBuilder.append(items.getYddh().substring(items.getYddh().length() - 4));
//                    be.setYddh(stringBuilder.toString());
//                } else if (items.getYddh().length() >= 4) {
//                    String result = items.getYddh().substring(2, items.getYddh().length() - 2);
//                    StringBuilder stringBuilder = new StringBuilder();
//                    stringBuilder.append(items.getYddh(), 0, 2);
//                    for (int i = 0; i < result.length(); i++) {
//                        stringBuilder.append("*");
//                    }
//                    stringBuilder.append(items.getYddh().substring(items.getYddh().length() - 2));
//                    be.setYddh(stringBuilder.toString());
//                }
//            }
//            newList.add(be);
//        }
//        jc.updateBatchById(newList);
//
//        //修改总部企业
//        List<BmsEnterpriseBasicInfo> qyList = jDataScreenMapper.getjh();
//        List<BmsEnterpriseBasicInfo> neList = new ArrayList<>();
//        for (BmsEnterpriseBasicInfo items : qyList) {
//            String result = items.getQymc().substring(2, items.getQymc().length()-4);
//            BmsEnterpriseBasicInfo be = new BmsEnterpriseBasicInfo();
//            be.setId(items.getId());
//            StringBuilder stringBuilder = new StringBuilder();
//            stringBuilder.append(items.getQymc(), 0, 2);
//            for (int i = 0; i < result.length(); i++) {
//                stringBuilder.append("*");
//            }
//            stringBuilder.append(items.getQymc().substring(items.getQymc().length()-4));
//            be.setQymc(stringBuilder.toString());
//            neList.add(be);
//        }
//        for (BmsEnterpriseBasicInfo i : neList) {
//            jDataScreenMapper.upjh(i);
//        }
//        return null;
//    }
}
